
import time, datetime
from mqttlogic import Expression

class AlarmConfig(object):

    def __init__(self, jsonObj, clientId):
        self.uuid = jsonObj['uuid']
        self.clientId = clientId
        self.enabled = (jsonObj.get('enabled', 1) == 1)
        trigger = jsonObj['trigger']

        after = trigger['afterTime'].split(':')
        before = trigger['beforeTime'].split(':')

        #time must be interpreted as gateway local time (in gateway's timezone)
        #get local timezone
        tz = datetime.datetime.now().astimezone().tzinfo

        self.afterTime = datetime.time(int(after[0]), int(after[1]), 0, 0, tz)
        self.beforeTime = datetime.time(int(before[0]), int(before[1]), 59, 0, tz)

        # for reed switches: triggers after a certain time period in which trigger condition must be met all the time
        self.triggerDelay = trigger.get('delay', 0) #in seconds

        hysteresis = trigger.get('hysteresis', 0.1)

        self.expression = Expression.Expression(trigger['expression'], hysteresis)

        #do not consider delay for integer/float expression, since hysteresis + delay can be complex
        if isinstance(self.expression.value, int) or isinstance(self.expression.value, float):
            self.triggerDelay = 0



    #return None if alarm should not trigger at all, return the delay when alarm should trigger otherwise
    def getTriggerDelay(self, mqttPath, jsonObj, ts):
        tz = datetime.datetime.now().astimezone().tzinfo

        #timestamp is unix time in ms
        nowTime = datetime.datetime.fromtimestamp(int(ts / 1000), tz).timetz()

        print ("now: " + str(nowTime) + ", start: " + str(self.afterTime) + ", end: " + str(self.beforeTime))


        if self.afterTime < self.beforeTime: # during day
            timeMatch = nowTime >= self.afterTime and nowTime <= self.beforeTime
        elif self.afterTime > self.beforeTime: #over night
            timeMatch = nowTime >= self.afterTime or nowTime <= self.beforeTime
        else:
            timeMatch = True

        #triggerAlarm = timeMatch and sensorId in self.sensorList and checkJsonAttributes(jsonObj) # and jsonReedStatus == self.reedStatus
        triggerAlarm = timeMatch and self.expression.evaluate(mqttPath, jsonObj)

        if not triggerAlarm:
            return None
        else:
            return self.triggerDelay
